# ----------------------------------------------------------------------------
# Output for HDD Health log files for RRDTool / Cacti
#
#   File: 		hddhealth.pl
#   Author:		BSOD2600 
#   Version:	1.2	
#   Date:		19/02/2005
#
#   Purpose:  This script reports HDD Health logfile values (aka S.M.A.R.T. data)
#			to rrdtool / Cacti. HDD Health is freeware and can be found 
#			here: http://www.panterasoft.com/
#
#	
#	Usage:	hddhealth.pl {HDD Report file} {HDD Data file}
#			The HDD report file is the name of the hard drive 
#			(ex: WDC WD740GD-75FLA1.txt).
#			The HDD data file is the serial number of the hard drive 
#			(ex: WD-WMAKE1359731.hdd).  You can find it either in the HDD 
#			report file or inside HDD Health.
#
#	Info:	This has been tested on WinXP/Win2003 with Perl 5.8.6.  
#			It should work on any Windows OS.
#
#			To grab files from remote computers use the switch: /n COMPUTER.
#			This assumes WinNT/2K/2K3 with Admin shares enabled.  
#			The User account running the scheduled service for 
#			cactid/poller.php must have administrator rights on the remote 
#			computer for it to work.  
#
#	History:
#		19-2-2005 1.2	Fixed problem with command line options & 
#						shorter report file names.
#		18-2-2005 1.1	Added ability to use remote computer
#		17-2-2005 1.0	Initial internal release
#
#-------- Don't edit below --------------------------------------------------
my ($err_num);

#-------- Detect which command line options were used -----------------------
if ($#ARGV < 1) {
	ERRORS(-1);
}

if ($#ARGV == 1) {
	$FileReport = $ARGV[0];
	$FileData = $ARGV[1];
	@FilePath = 'c:\\Program Files\\HDD Health\\';
}
elsif( $#ARGV == 2) {
	if ($ARGV[2] =~ /n/i) {
		if ($ARGV[3] eq "") {
				ERRORS(-2);
		}
	}
	$FileReport = "$ARGV[0] $ARGV[1]";
	$FileData = $ARGV[2];
	@FilePath = 'c:\\Program Files\\HDD Health\\';
}
elsif( $#ARGV == 3) {
	if ($ARGV[2] =~ /n/i) {
		if ($ARGV[3] eq "") {
				ERRORS(-2);
		}
	}
	elsif ($ARGV[3] =~ /n/i) {
		if ($ARGV[4] eq "") {
				ERRORS(-2);
		}
	}
	$FileReport = $ARGV[0];
	$FileData = $ARGV[1];
	$RemoteComp = $ARGV[3];
	@FilePath = "\\\\$RemoteComp\\c\$\\Program Files\\HDD Health\\";
}
elsif( $#ARGV == 4) {
	if ($ARGV[3] =~ /n/i) {
		if ($ARGV[4] eq "") {
				ERRORS(-2);
		}
	$FileReport = "$ARGV[0] $ARGV[1]";
	$FileData = $ARGV[2];
	$RemoteComp = $ARGV[4];
	@FilePath = "\\\\$RemoteComp\\c\$\\Program Files\\HDD Health\\";
	}
}

my ($rpr_start, $data_start, $data_stop, $name, $data);

#------- Open files and load data --------------------------------------------
open (RPT, "@FilePath$FileReport") or die "Can't open `$FileReport': $!";
@raw_rpt=<RPT>;
close(RPT);
open (DATA, "@FilePath$FileData") or die "Can't open `$FileData': $!";
@raw_data=<DATA>;
close(DATA);

#------- Search for start & stop of valid data -------------------------------
for ($i= 0; $i < @raw_data; $i++) {
        if ($raw_data[$i] =~ /AttrValues/i) {
			$data_start = $i-1;
		}
		if ($raw_data[$i] =~ /WarnValues/i) {
			$data_stop = $i;
			last;
		}
}
for ($j= 0; $j < @raw_rpt; $j++) {
        if ($raw_rpt[$j] =~ /S\.M\.A\.R\.T\. attribute/i) {
			$rpt_start = $j+2;
			last;
   		}
}

#------- Display HDD attributes ----------------------------------------------
for ($i= $data_start; $i < $data_stop; $i++) {
	if ($raw_rpt[$j] =~ /^\s*([*\w\/\s-]*?)\s*\d+/) {;
		$name = $1; 
		$name =~ s/\s//g;
		$name =~ s/\///g;
	}
	if ($raw_data[$i] =~ /(\d+)=(\d+)/) {;
		$data = $2;
	}
	if ($name ne "") {
		print("$name:$data ");
	}
	$j++;
}
#------ Error Messages ------------------------------------------------------
sub ERRORS {
	if ($_[0] == "-1") {
		print("\nUSAGE: hddhealth.pl {HDD Report file} {HDD Data file} [/n]\n\n");
		print("/n COMPUTER	Used to access a remote computers HDD Health files\n");
		print("		either name or IP valid.\n");
		exit(-1);
	}
	elsif ($_[0] == "-2") {
		print("ERROR: You did not specify a computer name or IP\n");
		exit(-2);
	}
	elsif ($_[0] == "-3") {
		print("ERROR: You did not use enough arguments\n");
		exit(-3);
	}
}